#!/bin/sh

(
printf "%b\n" 'Playing with margins/padding,^tag(root)'
printf "%b\n" 'submenu,^checkout(foo)'
printf "%b\n" 'terminal,xterm'
printf "%b\n" 'browser,firefox'
printf "%b\n" 'file manager,pcmanfm'
printf "%b\n" 'Try Scrolling/PgUp/PgDn/Home/End,^tag(foo)'
printf "%b\n" '^sep()'
printf "%b\n" '^sep()'
printf "%b\n" '..,^back()'
printf "%b\n" 'item1,xterm'
printf "%b\n" '^sep(separator with text)'
printf "%b\n" 'item2,xterm'
printf "%b\n" '^sep()'
printf "%b\n" '^sep()'
printf "%b\n" '^sep()'
printf "%b\n" '^sep()'
printf "%b\n" '^sep()'
printf "%b\n" '^sep()'
printf "%b\n" 'item3,xterm'
printf "%b\n" 'item4,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item5,xterm'
printf "%b\n" 'item6,xterm'
printf "%b\n" '^sep()'
printf "%b\n" '^sep(separator with text)'
printf "%b\n" 'item7,xterm'
printf "%b\n" 'item8,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item9,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item10,xterm'
printf "%b\n" 'item11,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item12,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item13,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item14,xterm'
printf "%b\n" 'item15,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item16,xterm'
printf "%b\n" 'item17,xterm'
printf "%b\n" 'item18,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item19,xterm'
printf "%b\n" 'item20,xterm'
printf "%b\n" 'item21,xterm'
printf "%b\n" 'item22,xterm'
printf "%b\n" 'item23,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item24,xterm'
printf "%b\n" 'item25,xterm'
printf "%b\n" 'item26,xterm'
printf "%b\n" 'item27,xterm'
printf "%b\n" '^sep()'
printf "%b\n" 'item28,xterm'
printf "%b\n" 'item29,xterm'
printf "%b\n" 'item30,xterm'
printf "%b\n" 'item31,xterm'
printf "%b\n" 'item32 (last),xterm'
) | ../jgmenu --simple --config-file=./ex02/jgmenurc
